/*
 *
 * IP SP00FiNG DETECTi0N ViA LiNUX NETW0RK LAYER
 * ---------------------------------------------
 *
 * This lkm detects a possible spoofed pkt from your host and gives you the
 * spoofed ip
 *
 * Compile with: gcc -O6 -c detect-spoof.c -I/usr/src/linux/include
 *
 * Usage: insmod detect-spoof ip=yourip [dev=eth0]
 *
 * If you have a box with more than one ip, you have to modify this module...
 * This code only checks if the source ip is different than 127.0.0.1 and
 * if ip=yourip .
 *
 * Based on the artice "Building Into The Linux Network Layer" written by
 * kossak and lifeline (Phrack vol.9, issue 55, file 12 of 19)
 *
 * pIGpEN <pigpen@s0ftpj.org>
 *
 */

#define MODULE
#define __KERNEL__

#include <linux/config.h>
#include <linux/module.h>
#include <linux/version.h>

#include <linux/byteorder/generic.h>
#include <linux/netdevice.h>
#include <net/protocol.h>
#include <net/pkt_sched.h>
#include <linux/if_ether.h>
#include <linux/ip.h>
#include <linux/skbuff.h>

#include <linux/kernel.h>
#include <linux/mm.h>
#include <linux/file.h>
#include <asm/uaccess.h>


char *dev,*ip;

MODULE_PARM(dev,  "s");
MODULE_PARM(ip,   "s");


struct packet_type s_proto;
struct device *d;

/* net/ipv4/utils.c */

char *in_ntoa(__u32 in) {
        static char buff[18];
        char *p;

        p = (char *) &in;
        sprintf(buff, "%d.%d.%d.%d",
                (p[0] & 255), (p[1] & 255), (p[2] & 255), (p[3] & 255));
        return(buff);
}

/* ditto */

__u32 in_aton(const char *str) {
        unsigned long l;
        unsigned int val;
        int i;

        l = 0;
        for (i = 0; i < 4; i++) {
                l <<= 8;
                if (*str != '\0') {
                        val = 0;
                        while (*str != '\0' && *str != '.') {
                                val *= 10;
                                val += *str - '0';
                                str++;
                        }
                        l |= val;
                        if (*str != '\0')
                                str++;
                }
        }
        return(htonl(l));
}


int otp_func(struct sk_buff *skb, struct device *dv, struct packet_type *pt) {

       //skb->h.raw = skb->nh.raw + skb->nh.iph->ihl*4;

  if(skb->pkt_type==PACKET_OUTGOING)
   {
    if(skb->nh.iph->saddr!=in_aton(ip) && 
       skb->nh.iph->saddr!=in_aton("127.0.0.1"))
    {
     printk(KERN_WARNING "Detect possible spoofing from your box");
     printk(KERN_WARNING "Spoofed ip = %s", in_ntoa(skb->nh.iph->saddr));
    }
   }
  return 1;   
}


int init_module() {
 if(!ip) {
  printk("Usage: insmod ip=x.x.x.x [dev=devname]\n\n");
  return -ENXIO;
  }

 if (dev) {
   d = dev_get(dev);
   if (!d) {
    printk("Did not find device %s!\n", dev);
    printk("Using all known devices...");
   }
   else {
    printk("Using device %s, ifindex: %i\n",
            dev, d->ifindex);
    s_proto.dev = d;
   }
 }
 else
   printk("Using all known devices(wildcarded)...\n");

   s_proto.type = htons(ETH_P_ALL);
   s_proto.func = otp_func;
   dev_add_pack(&s_proto);

   return(0);
}

void cleanup_module() {
    dev_remove_pack(&s_proto);
    printk("Module unloaded\n");
}
